'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var NodeCache = require('node-cache');
/* eslint no-unused-vars:0 */
var cloneDeep = require('lodash/cloneDeep');
var isObject = require('lodash/isObject');

function clone(object) {
  if ((typeof object === 'undefined' ? 'undefined' : _typeof(object)) === 'object' && object !== null) {
    return cloneDeep(object);
  }
  return object;
}

/**
 * Wrapper for node-cache.
 *
 * @param {object} args - Args passed to underlying lru-cache, plus additional optional args:
 * @param {boolean} [args.shouldCloneBeforeSet=true] - Whether to clone the data being stored.
 *   Default: true
 * @param {boolean} [args.usePromises=true] - Whether to enable the use of Promises. Default: true
 */
var nodeCacheStore = function nodeCacheStore(args) {
  args = args || {};
  var self = {};
  self.name = 'node-cache';
  var Promise = args.promiseDependency || global.Promise;
  self.usePromises = !(typeof Promise === 'undefined' || args.noPromises);
  self.shouldCloneBeforeSet = args.shouldCloneBeforeSet !== false; // clone by default

  var instanceOpts = {
    stdTTL: args.stdTTL,
    checkperiod: args.checkperiod,
    useClones: args.useClones,
    deleteOnExpire: args.deleteOnExpire,
    enableLegacyCallbacks: args.enableLegacyCallbacks,
    maxKeys: args.maxKeys
  };

  var cacheInstance = new NodeCache(instanceOpts);

  var setMultipleKeys = function setMultipleKeys(keysValues, maxAge) {
    var length = keysValues.length;
    var values = [];
    for (var i = 0; i < length; i += 2) {
      cacheInstance.set(keysValues[i], keysValues[i + 1], maxAge);
      values.push(keysValues[i + 1]);
    }
    return values;
  };

  self.set = function (key, value, options, cb) {
    if (self.shouldCloneBeforeSet) {
      value = clone(value);
    }

    if (typeof options === 'function') {
      cb = options;
      options = {};
    }
    options = options || {};

    var maxAge = options.ttl || options.ttl === 0 ? options.ttl * 1000 : instanceOpts.stdTTL;

    cacheInstance.set(key, value, maxAge);
    if (cb) {
      process.nextTick(cb.bind(null, null, true));
    } else if (self.usePromises) {
      return Promise.resolve(value);
    }
  };

  self.mset = function () {
    var args = Array.prototype.slice.apply(arguments);
    var cb = void 0;
    var options = {};

    if (typeof args[args.length - 1] === 'function') {
      cb = args.pop();
    }

    if (args.length % 2 > 0 && isObject(args[args.length - 1])) {
      options = args.pop();
    }

    var maxAge = options.ttl || options.ttl === 0 ? options.ttl * 1000 : instanceOpts.stdTTL;

    var values = setMultipleKeys(args, maxAge);

    if (cb) {
      process.nextTick(cb.bind(null, null));
    } else if (self.usePromises) {
      return Promise.resolve(values);
    }
  };

  self.get = function (key, options, cb) {
    if (typeof options === 'function') {
      cb = options;
    }
    var value = cacheInstance.get(key);

    if (cb) {
      process.nextTick(cb.bind(null, null, value));
    } else if (self.usePromises) {
      return Promise.resolve(value);
    } else {
      return value;
    }
  };

  self.mget = function () {
    var args = Array.prototype.slice.apply(arguments);
    var cb = void 0;
    var options = {};

    if (typeof args[args.length - 1] === 'function') {
      cb = args.pop();
    }

    if (isObject(args[args.length - 1])) {
      options = args.pop();
    }

    var values = args.map(function (key) {
      return cacheInstance.get(key);
    });

    if (cb) {
      process.nextTick(cb.bind(null, null, values));
    } else if (self.usePromises) {
      return Promise.resolve(values);
    } else {
      return values;
    }
  };

  self.del = function () {
    var args = Array.prototype.slice.apply(arguments);
    var cb = void 0;
    var options = {};

    if (typeof args[args.length - 1] === 'function') {
      cb = args.pop();
    }

    if (isObject(args[args.length - 1])) {
      options = args.pop();
    }

    if (Array.isArray(args[0])) {
      args = args[0];
    }

    args.forEach(function (key) {
      cacheInstance.del(key);
    });

    if (cb) {
      process.nextTick(cb.bind(null, null));
    } else if (self.usePromises) {
      return Promise.resolve();
    }
  };

  self.reset = function (cb) {
    cacheInstance.flushAll();
    if (cb) {
      process.nextTick(cb.bind(null, null));
    } else if (self.usePromises) {
      return Promise.resolve();
    }
  };

  self.keys = function (cb) {
    var keys = cacheInstance.keys();
    if (cb) {
      process.nextTick(cb.bind(null, null, keys));
    } else if (self.usePromises) {
      return Promise.resolve(keys);
    } else {
      return keys;
    }
  };

  self.ttl = function (key) {
    return cacheInstance.getTtl(key);
  };

  return self;
};

var methods = {
  create: function create(args) {
    return nodeCacheStore(args);
  }
};

module.exports = methods;